package sdanderson.androidcharts.BarChart;

import android.content.Context;
import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.drawable.BitmapDrawable;
import android.util.AttributeSet;
import android.util.TypedValue;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.LinearLayout;
import android.widget.PopupWindow;
import android.widget.RelativeLayout;
import android.widget.TextView;

import java.text.DecimalFormat;

import sdanderson.androidcharts.Common.NumberUtilities;
import sdanderson.androidcharts.R;

/**
 * Created by Shane Anderson on 10/4/2015.
 *
 * This is the ViewGroup of the bar chart plotting
 *
 */
public class BarChart extends RelativeLayout {

    private TextView mTextViewTitle;
    private TextView mTextViewYAxisLabel;
    private TextView mTextViewXAxisLabel;
    private TextView mTextViewYAxisLargestValue;
    private TextView mTextViewYAxisSmallestValue;
    private boolean mUseNumberAbbreviationSuffixYValueChart = false;
    private boolean mUseNumberAbbreviationSuffixYValuePopup = false;
    private DecimalFormat mDecimalFormatChartY = new DecimalFormat("#.#");
    private DecimalFormat mDecimalFormatPopupY = new DecimalFormat("#.#");
    private String mNumberSuffixes = "kmbt";
    private int mNumberGroupSplit = 3;

    PopupWindow mLingeringPopupWindow = null;

    //private View mV;
    private BarChartInnerView mBarChartInnerView;

    private OnBarTouchedListener mOnBarTouchedListener = null;

    public BarChart (Context pContext) {
        super(pContext);
        init(pContext);
    }

    public BarChart (Context pContext, AttributeSet pAttrs) {
        super(pContext, pAttrs);
        init(pContext);
    }

    private void init(Context pContext) {
        Paint paint;
        setBackgroundColor(Color.TRANSPARENT);
        paint = new Paint(Paint.ANTI_ALIAS_FLAG);
        paint.setStyle(Paint.Style.FILL);

        View.inflate(pContext, R.layout.barchartrelative, this);
        setLayoutParams(new LinearLayout.LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.MATCH_PARENT));
        mBarChartInnerView = (BarChartInnerView) findViewById(R.id.barchart_chart);
        mTextViewTitle = (TextView) findViewById(R.id.barchart_title);
        mTextViewTitle.setText(" ");
        mTextViewXAxisLabel = (TextView) findViewById(R.id.barchart_xaxis_label);
        mTextViewXAxisLabel.setText(" ");
        mTextViewYAxisLabel = (TextView) findViewById(R.id.barchart_yaxis_label);
        mTextViewYAxisLabel.setText(" ");
        mTextViewYAxisLargestValue = (TextView) findViewById(R.id.barchart_yaxishighestvalue);
        mTextViewYAxisSmallestValue = (TextView) findViewById(R.id.barchart_yaxislowestvalue);

        mBarChartInnerView.setOnTouchListener(new OnTouchListener() {
            float mX = 0, mY = 0;

            @Override
            public boolean onTouch(View view, MotionEvent motionEvent) {
                if (motionEvent.getAction() == MotionEvent.ACTION_DOWN) {
                    mX = motionEvent.getX();
                    mY = motionEvent.getY();
                }
                if (motionEvent.getEventTime() - motionEvent.getDownTime() < 500 &&
                        motionEvent.getAction() == MotionEvent.ACTION_UP) {
                    if (mOnBarTouchedListener != null) {
                        mOnBarTouchedListener.onBarTouched(mBarChartInnerView.tappedGetBarChartItem(mX, mY),
                                BarChart.this, mX, mY);
                    } else {
                        defaultTouchResponse(mBarChartInnerView.tappedGetBarChartItem(mX, mY), BarChart.this, mX, mY);
                    }
                }
                return true;
            }
        });
        //addView(mV);
    }

    @Override
    protected void onLayout (boolean pChanged, int pLeft, int pTop, int pRight, int pBottom) {
        super.onLayout(pChanged, pLeft, pTop, pRight, pBottom);
    }

    @Override
    protected void onSizeChanged (int pWidth, int pHeight, int pOldWidth, int pOldHeight) {
        super.onSizeChanged(pWidth, pHeight, pOldWidth, pOldHeight);
    }

    @Override
    protected void onDetachedFromWindow() {
        super.onDetachedFromWindow();
        if (mLingeringPopupWindow != null && mLingeringPopupWindow.isShowing()) {
            mLingeringPopupWindow.dismiss();
        }
    }

    private void defaultTouchResponse(BarChartItem pBar, BarChart pBarChart, float pTouchedX,
                                      float pTouchedY) {

        // If null, we clicked on the border or some other area of the bar chart that doesn't
        // represent a pie chart slice
        if (pBar != null) {
            LayoutInflater inflater = (LayoutInflater)
                    getContext().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
            View v = inflater.inflate(R.layout.piebarchart_popupwindow, (ViewGroup) getParent(), false);
            TextView tv = (TextView) v.findViewById(R.id.piebarchart_popupwindow_slicetitletext);
            tv.setText(pBar.getLabel());
            tv = (TextView) v.findViewById(R.id.piebarchart_popupwindow_slicetitlevalue);

            if (mUseNumberAbbreviationSuffixYValuePopup)
                tv.setText(NumberUtilities.formatValueWithSuffix(pBar.getValue(), mDecimalFormatPopupY,
                                                mNumberSuffixes, mNumberGroupSplit));
            else {
                tv.setText(mDecimalFormatPopupY.format(pBar.getValue()));
            }

            View colorkey = v.findViewById(R.id.piebarchart_popupwindow_colorkey);
            colorkey.setBackgroundColor(pBar.getColor());
            final PopupWindow pw = new PopupWindow(v, WindowManager.LayoutParams.WRAP_CONTENT,
                                        WindowManager.LayoutParams.WRAP_CONTENT);
            mLingeringPopupWindow = pw;
            // Need to set a background drawable to some random bitmap to make it touchable (bug?)
            pw.setBackgroundDrawable(new BitmapDrawable(getContext().getResources(),
                    Bitmap.createBitmap(1, 1, Bitmap.Config.ARGB_8888)));
            pw.setOutsideTouchable(true);
            pw.setTouchInterceptor(new View.OnTouchListener() {
                @Override
                public boolean onTouch(View view, MotionEvent motionEvent) {
                    pw.dismiss();
                    return true;
                }
            });
            View parentView = (View) getParent();
            int[] location = new int[2];
            pBarChart.getLocationOnScreen(location);
            Resources r = getResources();
            float pxY   = TypedValue.applyDimension(
                    TypedValue.COMPLEX_UNIT_DIP, 80, r.getDisplayMetrics());
            pw.showAtLocation(parentView, Gravity.NO_GRAVITY,
                    location[0]+(int)pTouchedX, location[1]+(int)pTouchedY-(int)(pxY));
        }
    }

    private void refreshAxisValueText() {
        if (mBarChartInnerView.areThereBars()) {
            if (mUseNumberAbbreviationSuffixYValueChart) {
                mTextViewYAxisSmallestValue.setText(
                        NumberUtilities.formatValueWithSuffix(
                                mBarChartInnerView.getYAxisSmallestValue(), mDecimalFormatChartY,
                                mNumberSuffixes, mNumberGroupSplit));
                mTextViewYAxisLargestValue.setText(
                        NumberUtilities.formatValueWithSuffix(
                                mBarChartInnerView.getYAxisLargestValue(), mDecimalFormatChartY,
                                mNumberSuffixes, mNumberGroupSplit));
            } else {
                mTextViewYAxisSmallestValue.setText(
                        mDecimalFormatChartY.format(mBarChartInnerView.getYAxisSmallestValue()));
                mTextViewYAxisLargestValue.setText(
                        mDecimalFormatChartY.format(mBarChartInnerView.getYAxisLargestValue()));
            }
        }
    }

    // PUBLICLY AVAILABLE INTERACTION METHODS

    @SuppressWarnings("unused")
    public void addBar(String pLabel, float pValue, int pColor) {
        mBarChartInnerView.addBar(pLabel, pValue, pColor);
        refreshAxisValueText();
    }

    @SuppressWarnings("unused")
    public BarChartItem getBarChartItemByColor(int pColor) {
        return mBarChartInnerView.getBarChartItemByColor(pColor);
    }

    @SuppressWarnings("unused")
    public void setAllBarBorderWidths(float pWidth) {
        mBarChartInnerView.setAllBarBorderWidths(pWidth);
    }

    @SuppressWarnings("unused")
    public void setAllBarBorderColors(int pColor) {
        mBarChartInnerView.setAllBarBorderColors(pColor);
    }

    @SuppressWarnings("unused")
    public void setChartTitle(String pTitle) { mTextViewTitle.setText(pTitle); }

    @SuppressWarnings("unused")
    public void setChartTitleColor(int pColor) { mTextViewTitle.setTextColor(pColor); }

    @SuppressWarnings("unused")
    public void setChartBackgroundColor(int pColor) {
        mBarChartInnerView.setChartBackgroundColor(pColor);
    }

    @SuppressWarnings("unused")
    public void setGridlineWidth(float pGridlineWidth) {
        mBarChartInnerView.setGridlineWidth(pGridlineWidth); }

    // This is to set the number of gridlines on both sides of the Y=0 line
    @SuppressWarnings("unused")
    public void setNumOfGridlines(int pNumOfGridlines) {
        mBarChartInnerView.setNumOfGridlines(pNumOfGridlines); }

    @SuppressWarnings("unused")
    public void setChartGridlineColor(int pColor) {
        mBarChartInnerView.setGridlineColor(pColor); }

    @SuppressWarnings("unused")
    public void setZeroLineWidth(float pWidth) {
        mBarChartInnerView.setZeroLineWidth(pWidth); }

    @SuppressWarnings("unused")
    public void setZeroLineColor(int pColor) { mBarChartInnerView.setZeroLineColor(pColor); }

    @SuppressWarnings("unused")
    public void setRoundOutChartYMinMax(boolean pRound) { setRoundOutChartYMinMax(pRound, 1); }

    @SuppressWarnings("unused")
    public void setRoundOutChartYMinMax(boolean pRound, int pHighestDigits) {
        mBarChartInnerView.setRoundOutChartYMinMax(pRound, pHighestDigits);
        refreshAxisValueText();
    }

    @SuppressWarnings("unused")
    public void setBorderWidth(float pWidth) { mBarChartInnerView.setBorderWidth(pWidth); }

    @SuppressWarnings("unused")
    public void setBorderColor(int pColor) { mBarChartInnerView.setBorderColor(pColor); }

    @SuppressWarnings("unused")
    public void setXLabelColor(int pColor) { mTextViewXAxisLabel.setTextColor(pColor); }

    @SuppressWarnings("unused")
    public void setYLabelColor(int pColor) { mTextViewYAxisLabel.setTextColor(pColor); }

    @SuppressWarnings("unused")
    public void setYValueLabelsColor(int pColor) {
        mTextViewYAxisLargestValue.setTextColor(pColor);
        mTextViewYAxisSmallestValue.setTextColor(pColor);
    }

    @SuppressWarnings("unused")
    public void setXLabel(String pLabel) {
        mTextViewXAxisLabel.setText(pLabel);
    }

    @SuppressWarnings("unused")
    public void setYLabel(String pLabel) {
        mTextViewYAxisLabel.setText(pLabel);
    }

    @SuppressWarnings("unused")
    public void setDecimalFormatChartY(DecimalFormat pDecimalFormat) {
        mDecimalFormatChartY = pDecimalFormat;
        refreshAxisValueText();
    }

    @SuppressWarnings("unused")
    public void setDecimalFormatPopupY(DecimalFormat pDecimalFormat) {
        mDecimalFormatPopupY = pDecimalFormat;
    }

    @SuppressWarnings("unused")
    public void setUseNumberAbbreviationSuffixYValueChart(boolean pUseSuffix) {
        mUseNumberAbbreviationSuffixYValueChart = pUseSuffix;
        refreshAxisValueText();
    }

    @SuppressWarnings("unused")
    public void setUseNumberAbbreviationSuffixYValuePopup(boolean pUseSuffix) {
        mUseNumberAbbreviationSuffixYValuePopup = pUseSuffix;
    }

    @SuppressWarnings("unused")
    public void setNumberSuffixes(String pSuffixes) {
        mNumberSuffixes = pSuffixes;
        refreshAxisValueText();
    }

    @SuppressWarnings("unused")
    public void setNumberGroupSplit(int pGroupSplit) {
        mNumberGroupSplit = pGroupSplit;
        refreshAxisValueText();
    }

    @SuppressWarnings("unused")
    public void setUseAnimation(boolean pUseAnimation) {
        mBarChartInnerView.setUseAnimation(pUseAnimation);
    }

    @SuppressWarnings("unused")
    public void setAnimationTime(float pAnimationTime) {
        mBarChartInnerView.setAnimationTime(pAnimationTime);
    }

    @SuppressWarnings("unused")
    public void reanimate() { mBarChartInnerView.reanimate(); }

    @SuppressWarnings("unused")
    public void setOnBarTouchedListener(OnBarTouchedListener pOnBarTouchedListener) {
        mOnBarTouchedListener = pOnBarTouchedListener;
    }

    @SuppressWarnings("unused")
    public interface OnBarTouchedListener {
        void onBarTouched(BarChartItem pBar, BarChart pPieChart, float pTouchedX, float pTouchedY);
    }
}
